@@ -22,24 +22,24 @@ module Text.PrettyPrint.Leijen.Text.Monadic (
 
    -- * Basic combinators
    empty, char, text, (<>), nest, line, linebreak, group, softline,
-   softbreak,
+   softbreak, spacebreak,
 
    -- * Alignment
    --
-   -- The combinators in this section can not be described by Wadler's
-   -- original combinators. They align their output relative to the
-   -- current output position - in contrast to @nest@ which always
-   -- aligns to the current nesting level. This deprives these
-   -- combinators from being \`optimal\'. In practice however they
-   -- prove to be very useful. The combinators in this section should
-   -- be used with care, since they are more expensive than the other
-   -- combinators. For example, @align@ shouldn't be used to pretty
-   -- print all top-level declarations of a language, but using @hang@
-   -- for let expressions is fine.
+   -- | The combinators in this section can not be described by Wadler's
+   --   original combinators. They align their output relative to the
+   --   current output position - in contrast to @nest@ which always
+   --   aligns to the current nesting level. This deprives these
+   --   combinators from being \`optimal\'. In practice however they
+   --   prove to be very useful. The combinators in this section should
+   --   be used with care, since they are more expensive than the other
+   --   combinators. For example, @align@ shouldn't be used to pretty
+   --   print all top-level declarations of a language, but using @hang@
+   --   for let expressions is fine.
    align, hang, indent, encloseSep, list, tupled, semiBraces,
 
    -- * Operators
-   (<+>), (<$>), (</>), (<$$>), (<//>),
+   (<+>), (<++>), (<$>), (</>), (<$$>), (<//>),
 
    -- * List combinators
    hsep, vsep, fillSep, sep, hcat, vcat, fillCat, cat, punctuate,
@@ -65,14 +65,15 @@ module Text.PrettyPrint.Leijen.Text.Monadic (
 
    -- * Rendering
    SimpleDoc(..), renderPretty, renderCompact, renderOneLine,
-   displayT, displayIO, putDoc, hPutDoc
+   displayB, displayT, displayIO, putDoc, hPutDoc
 
    ) where
 
 import qualified Text.PrettyPrint.Leijen.Text as PP
 import Text.PrettyPrint.Leijen.Text( Doc, SimpleDoc(..)
                                    , renderPretty, renderCompact, renderOneLine
-                                   , displayT, displayIO, putDoc, hPutDoc, Pretty(..))
+                                   , displayB, displayT, displayIO
+                                   , putDoc, hPutDoc, Pretty(..))
 
 import Data.String (IsString(fromString))
 import Control.Monad(liftM, liftM2, liftM3, liftM4)
@@ -80,7 +81,7 @@ import Data.Text.Lazy(Text)
 import Data.Int(Int64)
 
 infixr 5 </>,<//>,<$>,<$$>
-infixr 6 <>,<+>
+infixr 6 <>,<+>,<++>
 
 instance Monad m => IsString (m Doc) where
     fromString = string . fromString
@@ -121,13 +122,13 @@ semiBraces = liftM PP.semiBraces
 --   > list xs = encloseSep lbracket rbracket comma xs
 --   > test = text "list" <+> (list (map int [10,200,3000]))
 --
---   Which is layed out with a page width of 20 as:
+--   Which is laid out with a page width of 20 as:
 --
 --   @
 --   list [10,200,3000]
 --   @
 --
---   But when the page width is 15, it is layed out as:
+--   But when the page width is 15, it is laid out as:
 --
 --   @
 --   list [10
@@ -143,13 +144,13 @@ encloseSep = liftM4 PP.encloseSep
 --   > someText = map text ["words","in","a","tuple"]
 --   > test = parens (align (cat (punctuate comma someText)))
 --
---   This is layed out on a page width of 20 as:
+--   This is laid out on a page width of 20 as:
 --
 --   @
 --   (words,in,a,tuple)
 --   @
 --
---   But when the page width is 15, it is layed out as:
+--   But when the page width is 15, it is laid out as:
 --
 --   @
 --   (words,
@@ -172,7 +173,7 @@ sep :: (Monad m) => m [Doc] -> m Doc
 sep = liftM PP.sep
 
 -- | The document @(fillSep xs)@ concatenates documents @xs@
---   horizontally with @(\<+\>)@ as long as its fits the page, than
+--   horizontally with @(\<+\>)@ as long as its fits the page, then
 --   inserts a @line@ and continues doing that for all documents in
 --   @xs@.
 --
@@ -193,7 +194,7 @@ hsep = liftM PP.hsep
 --   >
 --   > test = text "some" <+> vsep someText
 --
---   This is layed out as:
+--   This is laid out as:
 --
 --   @
 --   some text
@@ -227,7 +228,7 @@ cat :: (Monad m) => m [Doc] -> m Doc
 cat = liftM PP.cat
 
 -- | The document @(fillCat xs)@ concatenates documents @xs@
---   horizontally with @(\<\>)@ as long as its fits the page, than
+--   horizontally with @(\<\>)@ as long as its fits the page, then
 --   inserts a @linebreak@ and continues doing that for all documents
 --   in @xs@.
 --
@@ -253,10 +254,15 @@ vcat = liftM PP.vcat
 (<>) = liftM2 (PP.<>)
 
 -- | The document @(x \<+\> y)@ concatenates document @x@ and @y@ with
---   a @space@ in between.  (infixr 6)
+--   a 'space' in between.  (infixr 6)
 (<+>) :: (Monad m) => m Doc -> m Doc -> m Doc
 (<+>) = liftM2 (PP.<+>)
 
+-- | The document @(x \<++\> y)@ concatenates document @x@ and @y@ with
+--   a 'spacebreak' in between.  (infixr 6)
+(<++>) :: (Monad m) => m Doc -> m Doc -> m Doc
+(<++>) = liftM2 (PP.<++>)
+
 -- | The document @(x \<\/\> y)@ concatenates document @x@ and @y@
 --   with a 'softline' in between. This effectively puts @x@ and @y@
 --   either next to each other (with a @space@ in between) or
@@ -277,7 +283,7 @@ vcat = liftM PP.vcat
 (<$>) = liftM2 (PP.<$>)
 
 -- | The document @(x \<$$\> y)@ concatenates document @x@ and @y@
---   with a @linebreak@ in between. (infixr 5)
+--   with a 'linebreak' in between. (infixr 5)
 (<$$>) :: (Monad m) => m Doc -> m Doc -> m Doc
 (<$$>) = liftM2 (PP.<$$>)
 
@@ -291,6 +297,11 @@ softline = return PP.softline
 softbreak :: (Monad m) => m Doc
 softbreak = return PP.softbreak
 
+-- | The document @spacebreak@ behaves like 'space' when rendered normally
+-- but like 'empty' when using 'renderCompact' or 'renderOneLine'.
+spacebreak :: (Monad m) => m Doc
+spacebreak = return PP.spacebreak
+
 -- | Document @(squotes x)@ encloses document @x@ with single quotes
 --   \"'\".
 squotes :: (Monad m) => m Doc -> m Doc
@@ -446,7 +457,7 @@ rational = return . PP.rational
 prettyM :: (Pretty a, Monad m) => a -> m Doc
 prettyM = return . pretty
 
--- | The document @(fill i x)@ renders document @x@. It than appends
+-- | The document @(fill i x)@ renders document @x@. It then appends
 --   @space@s until the width is equal to @i@. If the width of @x@ is
 --   already larger, nothing is appended. This combinator is quite
 --   useful in practice to output a list of bindings. The following
@@ -461,7 +472,7 @@ prettyM = return . pretty
 --   >
 --   > test = text "let" <+> align (vcat (map ptype types))
 --
---   Which is layed out as:
+--   Which is laid out as:
 --
 --   @
 --   let empty  :: Doc
@@ -476,7 +487,7 @@ width :: (Monad m) => m Doc -> m (Int -> Doc) -> m Doc
 width = liftM2 PP.width
 
 -- | The document @(fillBreak i x)@ first renders document @x@. It
---   than appends @space@s until the width is equal to @i@. If the
+--   then appends @space@s until the width is equal to @i@. If the
 --   width of @x@ is already larger than @i@, the nesting level is
 --   increased by @i@ and a @line@ is appended. When we redefine
 --   @ptype@ in the previous example to use @fillBreak@, we get a
@@ -545,7 +556,7 @@ hang = liftM . PP.hang
 --
 --   > test = text "hi" <+> (text "nice" $$ text "world")
 --
---   which will be layed out as:
+--   which will be laid out as:
 --
 --   @
 --   hi nice
@@ -575,7 +586,8 @@ text = return . PP.text
 
 -- | The @line@ document advances to the next line and indents to the
 --   current nesting level. Document @line@ behaves like @(text \"
---   \")@ if the line break is undone by 'group'.
+--   \")@ if the line break is undone by 'group' or if rendered with
+--   'renderOneLine'.
 line :: (Monad m) => m Doc
 line = return PP.line
 
